@@ -1,4 +1,4 @@
-#include <uwsgi.h>
+#include "uwsgi.h"
 #include <openssl/rand.h>
 #include <openssl/sha.h>
 #include <openssl/md5.h>
@@ -231,6 +231,10 @@ SSL_CTX *uwsgi_ssl_new_server_context(char *name, char *crt, char *key, char *ci
 		ssloptions |= SSL_OP_NO_SSLv3;
 	}
 
+	if (!uwsgi.tlsv1) {
+		ssloptions |= SSL_OP_NO_TLSv1;
+	}
+
 // release/reuse buffers as soon as possibile
 #ifdef SSL_MODE_RELEASE_BUFFERS
         SSL_CTX_set_mode(ctx, SSL_MODE_RELEASE_BUFFERS);
@@ -326,8 +330,7 @@ SSL_CTX *uwsgi_ssl_new_server_context(char *name, char *crt, char *key, char *ci
                 else {
                         SSL_CTX_set_verify(ctx, SSL_VERIFY_PEER, uwsgi_ssl_verify_callback);
                 }
-                // in the future we should allow to set the verify depth
-                SSL_CTX_set_verify_depth(ctx, 1);
+                SSL_CTX_set_verify_depth(ctx, uwsgi.ssl_verify_depth);
 
 		if (uwsgi.ssl_tmp_dir && !uwsgi_starts_with(client_ca, strlen(client_ca), "-----BEGIN ", 11)) {
 			if (!name) {
